<?php

namespace FiloBlu\ExtInventory\Plugin\Magento\Sales\Api;

use Exception;
use FiloBlu\ExtInventory\Api\Data\InventoryFbOrderitemSourceInterface;
use FiloBlu\ExtInventory\Api\InventoryFbOrderitemSourceRepositoryInterface;
use FiloBlu\ExtInventory\Model\ShippingAlgorithms\SourceSlots\Algorithm;
use FiloBlu\ExtInventory\Model\ShippingAlgorithms\SourceSlots\SourcesData;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\InventorySales\Model\ResourceModel\StockIdResolver;
use Magento\InventorySalesApi\Api\Data\SalesChannelInterface;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Store\Model\StoreManagerInterface;
use Psr\Log\LoggerInterface;

/**
 * Class OrderManagementInterface
 * @package FiloBlu\ExtInventory\Plugin\Magento\Sales\Api
 */
class OrderManagementInterface
{
    /**
     * @var Algorithm
     */
    public $algorithm;
    /**
     * @var InventoryFbOrderitemSourceInterface
     */
    public $InventoryFbOrderItemSourceInterface;
    /**
     * @var InventoryFbOrderitemSourceRepositoryInterface
     */
    public $InventoryFbOrderItemSourceRepositoryInterface;
    /**
     * @var array
     */
    public $sourcesListByCode;
    /**
     * @var SourcesData
     */
    public $SourcesData;
    /**
     * @var StockIdResolver
     */
    private $stockIdResolver;
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;
    /**
     * @var LoggerInterface
     */
    private $logger;
    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * OrderManagementInterface constructor.
     * @param Algorithm $algorithm
     * @param InventoryFbOrderitemSourceRepositoryInterface $InventoryFbOrderItemSourceRepositoryInterface
     * @param InventoryFbOrderitemSourceInterface $InventoryFbOrderItemSourceInterface
     * @param StockIdResolver $stockIdResolver
     * @param StoreManagerInterface $storeManager
     * @param SourcesData $SourcesData
     * @param LoggerInterface $logger
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        Algorithm $algorithm,
        InventoryFbOrderitemSourceRepositoryInterface $InventoryFbOrderItemSourceRepositoryInterface,
        InventoryFbOrderitemSourceInterface $InventoryFbOrderItemSourceInterface,
        StockIdResolver $stockIdResolver,
        StoreManagerInterface $storeManager,
        SourcesData $SourcesData,
        LoggerInterface $logger,
        ScopeConfigInterface $scopeConfig
    ) {
        $this->algorithm = $algorithm;
        $this->InventoryFbOrderItemSourceRepositoryInterface = $InventoryFbOrderItemSourceRepositoryInterface;
        $this->InventoryFbOrderItemSourceInterface = $InventoryFbOrderItemSourceInterface;
        $this->SourcesData = $SourcesData;
        $this->stockIdResolver = $stockIdResolver;
        $this->storeManager = $storeManager;
        $this->logger = $logger;
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * @param \Magento\Sales\Api\OrderManagementInterface $subject
     * @param OrderInterface $order
     * @return OrderInterface
     */
    public function afterPlace(\Magento\Sales\Api\OrderManagementInterface $subject, $order)
    {
        $enabled = $this->scopeConfig->getValue('msi/general/enable');

        if (!$enabled) {
            return $order;
        }

        try {
            $websiteId = $this->storeManager->getStore($order->getStoreId())->getWebsiteId();
            $websiteCode = $this->storeManager->getWebsite($websiteId)->getCode();

            $stockId = $this->stockIdResolver->resolve(SalesChannelInterface::TYPE_WEBSITE, $websiteCode);
            $algorithmPreview = $this->algorithm->getResultPreviewFromOrder($order, $stockId);
            $this->sourcesListByCode = array_flip($this->SourcesData->getSourcesArrayById($stockId));

            // Algorithm returned zero results
            if (count($algorithmPreview) === 0) {
                return $order;
            }

            $bestSolution = $algorithmPreview[0];
            unset($bestSolution['_points']); //Removing debug data

            // Finding how many sources are involved
            $sources_involved = [];
            foreach ($bestSolution as $source_code => $items) {
                foreach ($items as $sku => $qty) {
                    if ($qty > 0) {
                        $sources_involved[] = $source_code;
                    }
                }
            }
            $sources_involved = array_values(array_unique($sources_involved));

            if (count($sources_involved) === 1) {
                $final_source_id = $this->sourcesListByCode[$sources_involved[0]];
            } else {
                $final_source_id = $this->sourcesListByCode[$this->SourcesData->getMainSourceCode()];
            }

            foreach ($bestSolution as $source_code => $items) {
                if (!isset($this->sourcesListByCode[$source_code])) {
                    // source_code not found, do what? throw error?
                    $this->logger->error("Source_code {$source_code} not found");
                }
                // Adding data on the support table (only if this is a 'mulstistock' order?)
                foreach ($items as $sku => $qty) {
                    if ($qty < 1) {
                        continue;
                    }
                    $orderItem = $this->findOrderItemFromSku($order, $sku);
                    $InventoryFbOrderitemSourceInterface = $this->InventoryFbOrderItemSourceInterface;
                    $InventoryFbOrderitemSourceInterface->setOrderId($order->getEntityId());
                    $InventoryFbOrderitemSourceInterface->setOrderItemId($orderItem->getItemId());
                    $InventoryFbOrderitemSourceInterface->setInventorySourceId($this->sourcesListByCode[$source_code]);
                    $InventoryFbOrderitemSourceInterface->setFinalInventorySourceId($final_source_id);
                    $InventoryFbOrderitemSourceInterface->setSku($sku);
                    $InventoryFbOrderitemSourceInterface->setQty($qty);
                    $this->InventoryFbOrderItemSourceRepositoryInterface->save($InventoryFbOrderitemSourceInterface);
                }
            }
        } catch (Exception $e) {
            $this->logger->error($e->getMessage(), ['exception' => $e]);
        }

        return $order;
    }

    public function findOrderItemFromSku($order, $simple_sku)
    {
        $items = $order->getAllItems();
        foreach ($items as $item) {
            if (!$item->getParentItem() && $item->getSku() === $simple_sku) {
                return $item;
            }
        }
        return false;
    }
}
